package com.loit.mybatis.example.service.impl;

import com.alicp.jetcache.Cache;
import com.alicp.jetcache.RefreshPolicy;
import com.alicp.jetcache.anno.CacheType;
import com.alicp.jetcache.anno.CreateCache;
import com.loit.mybatis.example.entity.TStorage;
import com.loit.mybatis.example.service.ITStorageCacheService;
import com.loit.mybatis.example.service.ITStorageService;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.concurrent.TimeUnit;

@Service("iTStorageCacheService")
public class ITStorageCacheServiceImpl implements ITStorageCacheService {

    @Resource(name = "iTStorageService")
    private ITStorageService itStorageService;

    @CreateCache(expire = 100, cacheType = CacheType.REMOTE)
    private Cache<Integer, TStorage> storageCache;

    @Override
    public TStorage getStorageCache(Integer storageId) {
        return storageCache.get(storageId);
    }

    @Override
    public void removeCache(Integer id) {
        storageCache.remove(id);
    }

    @PostConstruct
    public void init() {
        // 自动刷新
        RefreshPolicy policy = RefreshPolicy.newPolicy(5, TimeUnit.MINUTES).stopRefreshAfterLastAccess(30, TimeUnit.MINUTES);
        storageCache.config().setLoader(this::loadMenuByUserFromDatabase);
        storageCache.config().setRefreshPolicy(policy);
    }

    private TStorage loadMenuByUserFromDatabase(Integer id) {
        return itStorageService.getById(id);
    }

}
