package com.loit.mybatis.example.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.loit.mybatis.example.dto.CommodityDTO;
import com.loit.mybatis.example.entity.TStorage;
import com.loit.mybatis.example.enums.RspStatusEnum;
import com.loit.mybatis.example.mapper.TStorageMapper;
import com.loit.mybatis.example.response.ObjectResponse;
import com.loit.mybatis.example.service.ITStorageService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 库存服务实现类
 * </p>
 */
@Service(value = "iTStorageService")
@Transactional(readOnly = true)
@Slf4j
public class TStorageServiceImpl extends ServiceImpl<TStorageMapper, TStorage> implements ITStorageService {

    @Override
    @Transactional()
    public ObjectResponse decreaseStorage(CommodityDTO commodityDTO) {
        int storage = baseMapper.decreaseStorage(commodityDTO.getCommodityCode(), commodityDTO.getCount());
        ObjectResponse<Object> response = new ObjectResponse<>();


        if (storage > 0) {
            response.setMessage(RspStatusEnum.SUCCESS.getMessage());
            return response;
        }

        response.setStatus(RspStatusEnum.FAIL.getCode());
        response.setMessage(RspStatusEnum.FAIL.getMessage());

        return response;
    }


    @Override
    public List<TStorage> findAll() {

//        方式一:
//        List<TStorage> allStorage = this.list(null);

//        方式二:
        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();
        List<TStorage> allStorage = this.list(queryWrapper);

//        方式三:
//        TStorage storage = new TStorage();
//        List<TStorage> allStorage = storage.selectAll();

        return allStorage;
    }

    @Override
    public List<TStorage> findPage(Long pageNum, Long pageSize) {

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();
        Page<TStorage> page = new Page<>(pageNum, pageSize);
        //分页数据
        IPage<TStorage> pageInfo = this.page(page, queryWrapper);
        return pageInfo.getRecords();

    }

}
