package com.loit.mybatis.example.dao;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.loit.mybatis.example.entity.TStorage;
import com.loit.mybatis.example.mapper.TStorageMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RunWith(SpringRunner.class)
@SpringBootTest
@Slf4j
public class SimpleTest {

    @Autowired
    private TStorageMapper tStorageMapper;

    @Test
    public void select() {
        List<TStorage> storageList = tStorageMapper.selectList(null);

        log.info(storageList.toString());
    }


    @Test
    public void insert() {
        TStorage storage = new TStorage();

        storage.setName("pod1").setCount(9999).setCommodityCode("0002");

        int count = tStorageMapper.insert(storage);

        log.info(String.valueOf(count));
    }

    @Test
    public void selectById() {
        TStorage storage = tStorageMapper.selectById(1);
        log.info(storage.toString());
    }


    @Test
    public void selectByIds() {
        List<Integer> ids = Arrays.asList(1, 2);

        List<TStorage> storageList = tStorageMapper.selectBatchIds(ids);
        log.info(storageList.toString());
    }


    @Test
    public void selectByMap() {

        Map<String, Object> columnMap = new HashMap<>();
        columnMap.put("commodity_code", "0002");
        columnMap.put("name", "pod1");

        List<TStorage> storageList = tStorageMapper.selectByMap(columnMap);
        log.info(storageList.toString());

    }


    /**
     * 测试条件构造器
     * DEBUG==>  Preparing: SELECT id,commodity_code,name,count FROM t_storage WHERE commodity_code LIKE ? AND count < ?
     * DEBUG==> Parameters: C%(String), 888(Integer)
     */
    @Test
    public void selectByWrapper() {

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();
        queryWrapper.likeRight("commodity_code", "C");
        //queryWrapper.like("commodity_code", "C");
        queryWrapper.lt("count", 888);

        List<TStorage> storageList = tStorageMapper.selectList(queryWrapper);
        log.info(storageList.toString());

    }

    /**
     * DEBUG==>  Preparing: SELECT id,commodity_code,name,count FROM t_storage WHERE commodity_code LIKE ? OR count < ? ORDER BY commodity_code DESC , id ASC
     * DEBUG==> Parameters: C%(String), 888(Integer)
     */
    @Test
    public void selectByWrapperTwo() {

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();
        queryWrapper.likeRight("commodity_code", "C").or();
        //queryWrapper.like("commodity_code", "C");
        queryWrapper.lt("count", 888).orderByDesc("commodity_code").orderByAsc("id");

        List<TStorage> storageList = tStorageMapper.selectList(queryWrapper);
        log.info(storageList.toString());

    }

    /**
     * DEBUG==>  Preparing: SELECT id,commodity_code,name,count FROM t_storage WHERE date_format(create_time,'%Y-%m-%d') = ?
     * DEBUG==> Parameters: 2020-02-10(String)
     */
    @Test
    public void selectByWrapperThree() {

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();
        queryWrapper.apply("date_format(create_time,'%Y-%m-%d') = {0}", "2020-02-10");
        //queryWrapper.inSql("commodity_code","select commodity_code from xxxx where ");

        List<TStorage> storageList = tStorageMapper.selectList(queryWrapper);
        log.info(storageList.toString());

    }


    /**
     * SELECT id,commodity_code,name,count FROM t_storage WHERE commodity_code LIKE ? AND ( count < ? ) ORDER BY commodity_code DESC , id ASC
     */
    @Test
    public void selectByWrapperFour() {

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();
        queryWrapper.likeRight("commodity_code", "C").and(qw -> qw.lt("count", 888));
        queryWrapper.orderByDesc("commodity_code").orderByAsc("id");

        List<TStorage> storageList = tStorageMapper.selectList(queryWrapper);
        log.info(storageList.toString());

    }


    /**
     * SELECT id,commodity_code,name,count FROM t_storage WHERE ( commodity_code LIKE ? OR count < ? ) AND id = ? ORDER BY commodity_code DESC , id ASC
     */
    @Test
    public void selectByWrapperFive() {

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();
        queryWrapper.nested(qw -> qw.likeRight("commodity_code", "C").or().lt("count", 888));
        queryWrapper.eq("id", 2);
        queryWrapper.orderByDesc("commodity_code").orderByAsc("id");

        List<TStorage> storageList = tStorageMapper.selectList(queryWrapper);
        log.info(storageList.toString());

    }


    /**
     * DEBUG==>  Preparing: SELECT id,commodity_code FROM t_storage WHERE id IN (?,?) limit 1
     * DEBUG==> Parameters: 1(Integer), 2(Integer)
     */
    @Test
    public void selectByWrapperSix() {

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", Arrays.asList(1, 2)).last(" limit 1");

        List<TStorage> storageList = tStorageMapper.selectList(queryWrapper);
        log.info(storageList.toString());

    }


    @Test
    public void selectByWrapperSeven() {

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("id", "commodity_code").in("id", Arrays.asList(1, 2)).last(" limit 1");

        List<TStorage> storageList = tStorageMapper.selectList(queryWrapper);
        log.info(storageList.toString());

    }


    /**
     * DEBUG==>  Preparing: SELECT id,commodity_code,count FROM t_storage WHERE id IN (?,?) limit 1
     * DEBUG==> Parameters: 1(Integer), 2(Integer)
     */
    @Test
    public void selectByWrapperEight() {

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", Arrays.asList(1, 2)).last(" limit 1");
        queryWrapper.select(TStorage.class, info -> !info.getColumn().equals("create_time") && !info.getColumn().equals("name"));
        List<TStorage> storageList = tStorageMapper.selectList(queryWrapper);
        log.info(storageList.toString());

    }


    /**
     * 条件表达式
     * DEBUG==>  Preparing: SELECT id,commodity_code,name,count FROM t_storage WHERE 1=1 ORDER BY commodity_code DESC , id ASC
     * DEBUG==> Parameters:
     */
    @Test
    public void selectByWrapperNight() {

        String commodityCode = "";

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();
        queryWrapper.likeRight(StringUtils.isNotEmpty(commodityCode), "commodity_code", "C");
        queryWrapper.orderByDesc("commodity_code").orderByAsc("id");

        List<TStorage> storageList = tStorageMapper.selectList(queryWrapper);
        log.info(storageList.toString());

    }


    /**
     * DEBUG==>  Preparing: SELECT commodity_code,name,count,id FROM t_storage WHERE commodity_code=? AND 1=1 ORDER BY commodity_code DESC , id ASC
     * DEBUG==> Parameters: 0002(String)
     *
     * @TableField(condition = SqlCondition.LIKE)
     * DEBUG==>  Preparing: SELECT commodity_code,name,count,id FROM t_storage WHERE commodity_code LIKE CONCAT('%',?,'%') AND 1=1 ORDER BY commodity_code DESC , id ASC
     * DEBUG==> Parameters: 0002(String)
     */
    @Test
    public void selectByWrapperEntity() {


        TStorage whereStorage = new TStorage();
        whereStorage.setCommodityCode("0002");

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>(whereStorage);
        queryWrapper.orderByDesc("commodity_code").orderByAsc("id");

        List<TStorage> storageList = tStorageMapper.selectList(queryWrapper);
        log.info(storageList.toString());

    }


    /**
     * DEBUG==>  Preparing: SELECT id,commodity_code,name,count FROM t_storage WHERE commodity_code = ? AND name IS NULL
     * DEBUG==> Parameters: 0002(String)
     * <p>
     * queryWrapper.allEq(params, false);
     * <p>
     * DEBUG==>  Preparing: SELECT id,commodity_code,name,count FROM t_storage WHERE commodity_code = ?
     * DEBUG==> Parameters: 0002(String)
     */
    @Test
    public void selectByWrapperAllEq() {

        Map<String, Object> params = new HashMap<>();
        params.put("commodity_code", "0002");
        params.put("name", null);

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();
        //queryWrapper.allEq(params);
        //queryWrapper.allEq(params, true);
        //queryWrapper.allEq(params, false);

        queryWrapper.allEq((k, v) -> !k.equals("commodity_code"), params, false);
        List<TStorage> storageList = tStorageMapper.selectList(queryWrapper);
        log.info(storageList.toString());

    }


    /**
     * DEBUG==>  Preparing: SELECT commodity_code FROM t_storage WHERE commodity_code = ?
     * DEBUG==> Parameters: 0002(String)
     */
    @Test
    public void selectByWrapperSelectMap() {

        Map<String, Object> params = new HashMap<>();
        params.put("commodity_code", "0002");
        params.put("name", null);

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();

        queryWrapper.allEq(params, false).select("commodity_code");
        List<Map<String, Object>> maps = tStorageMapper.selectMaps(queryWrapper);
        log.info(maps.toString());

    }


    /**
     * DEBUG==>  Preparing: SELECT avg(count) avg1,avg(count) avg2 FROM t_storage WHERE 1=1 GROUP BY commodity_code HAVING sum(count) <?
     * DEBUG==> Parameters: 999999(Integer)
     */
    @Test
    public void selectByWrapperSelectMapTwo() {

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();

        queryWrapper.select("avg(count) avg1", "avg(count) avg2");
        queryWrapper.groupBy("commodity_code").having("sum(count) <{0}", 999999);
        List<Map<String, Object>> maps = tStorageMapper.selectMaps(queryWrapper);
        log.info(maps.toString());

    }


    /**
     * selectObjs 是有一列数据
     */
    @Test
    public void selectByWrapperSelectMapObject() {

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>();

        queryWrapper.select("avg(count) avg1", "avg(count) avg2");
        queryWrapper.groupBy("commodity_code").having("sum(count) <{0}", 999999);
        List<Object> maps = tStorageMapper.selectObjs(queryWrapper);
        log.info(maps.toString());

    }


    /**
     * DEBUG==>  Preparing: SELECT COUNT(1) FROM t_storage WHERE commodity_code LIKE CONCAT('%',?,'%')
     * DEBUG==> Parameters: 0002(String)
     */
    @Test
    public void selectByWrapperSelectCount() {

        TStorage whereStorage = new TStorage();
        whereStorage.setCommodityCode("0002");

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>(whereStorage);

        Integer integer = tStorageMapper.selectCount(queryWrapper);
        log.info(String.valueOf(integer));

    }


    @Test
    public void selectByWrapperSelectOne() {

        TStorage whereStorage = new TStorage();
        whereStorage.setCommodityCode("0002");

        QueryWrapper<TStorage> queryWrapper = new QueryWrapper<>(whereStorage);

        TStorage storage = tStorageMapper.selectOne(queryWrapper);
        log.info(storage.toString());

    }


    /**
     * DEBUG==>  Preparing: SELECT id,commodity_code,name,count FROM t_storage WHERE commodity_code LIKE ?
     * DEBUG==> Parameters: %C%(String)
     */
    @Test
    public void selectByWrapperSelectLambda() {

        LambdaQueryWrapper<TStorage> lambda = new QueryWrapper<TStorage>().lambda();
        // LambdaQueryWrapper<TStorage> tStorageLambdaQueryWrapper = new LambdaQueryWrapper<>();

        lambda.like(TStorage::getCommodityCode, "C");

        List<TStorage> storageList = tStorageMapper.selectList(lambda);
        log.info(storageList.toString());
    }


    /**
     * 自定义
     *
     * @Select("select * from t_storage ${ew.customSqlSeg}")
     * List<TStorage> selectAll(@Param(Constants.WRAPPER) Wrapper<TStorage> wrapper);
     */
//    @Test
//    public void selectByWrapperSelectMy() {
//
//        LambdaQueryWrapper<TStorage> lambda = new QueryWrapper<TStorage>().lambda();
//        lambda.like(TStorage::getCommodityCode, "C");
//
//        List<TStorage> storageList = tStorageMapper.selectAll(lambda);
//        log.info(storageList.toString());
//    }


}
